// This class was created by JaTiTV.

package net.t2code.t2codelib.BUNGEE.api.yaml;

import net.md_5.bungee.config.ConfigurationProvider;
import net.md_5.bungee.config.YamlConfiguration;
import net.t2code.t2codelib.util.T2C_ConfigItem;
import net.md_5.bungee.config.Configuration;
import net.t2code.t2codelib.util.T2C_LanguageEnum;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

public class T2CB_ConfigWriter {

    private static Configuration config;


    public static void createConfig(File configFile, T2C_ConfigItem[] values, String... header) {
        boolean exist = configFile.exists();
        if (!exist) {
            configFile.getParentFile().mkdirs();
            try {
                configFile.createNewFile();
            } catch (IOException e) {
                e.printStackTrace();
                return;
            }
        }

        try {
            config = ConfigurationProvider.getProvider(YamlConfiguration.class).load(configFile);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        Map<String, List<String>> comments = new LinkedHashMap<>();

        T2C_LanguageEnum lang = T2C_LanguageEnum.english;

        for (T2C_ConfigItem item : values) {
            if (item.getForceSet() || !exist) {
                if (!config.contains(item.getPath())) {
                    config.set(item.getPath(), item.getValue());
                }
                List<String> commandList = item.getComments().get(lang);
                if (commandList != null) {
                    comments.put(item.getPath(), commandList);
                }
            }
        }
        saveConfigWithComments(configFile, comments, header);
        readConfig(config, values);
    }

    private static void readConfig(Configuration config, T2C_ConfigItem[] values) {
        for (T2C_ConfigItem item : values) {
            item.setValue(config.get(item.getPath()));
        }
    }

    private static void saveConfigWithComments(File file, Map<String, List<String>> comments, String... headers) {
        try {
            StringBuilder configContent = new StringBuilder();
            for (String h : headers) {
                configContent.append("# ").append(h).append("\n");
            }
            configContent.append("\n");
            addSection(config, comments, configContent, "", 0);

            // Write the content to the file
            Files.write(file.toPath(), configContent.toString().getBytes());
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private static void addSection(Configuration section, Map<String, List<String>> comments, StringBuilder builder, String prefix, int indentLevel) {
        String indent = "  ".repeat(indentLevel);

        for (String key : section.getKeys()) {
            String fullKey = prefix.isEmpty() ? key : prefix + "." + key;
            Object value = section.get(key);

            // Add comment if it exists for this key
            List<String> commentList = comments.get(fullKey);
            if (commentList != null) {
                for (String c : commentList) {
                    if (c.isEmpty()) {
                        builder.append(indent).append(c).append("\n");
                    } else builder.append(indent).append("# ").append(c).append("\n");
                }
            }

            // Check if the value is a section (nested map)

            if (value instanceof Configuration) {
                // Correctly add the section
                builder.append(indent).append(key).append(":\n");
                addSection((Configuration) value, comments, builder, fullKey, indentLevel + 1);
            } else {
                // Add value with proper indentation
                //  builder.append(indent).append(key).append(": ").append(value).append("\n");
                if (value instanceof List) {
                    List<?> list = (List<?>) value;
                    if (list.size() == 1){
                        for (Object s : list) {
                            if (s instanceof String) {
                                builder.append(indent).append(key).append(": \"").append(value).append("\"\n");
                            } else builder.append(indent).append(key).append(": ").append(value).append("\n");
                        }
                    } else {
                        builder.append(indent).append(key).append(": ").append("\n");
                        for (Object s : list) {
                            if (s instanceof String) {
                                builder.append(indent).append("- \"").append(s).append("\"\n");
                            } else builder.append(indent).append("- ").append(s).append("\n");
                        }
                    }
                } else {
                    if (value instanceof String) {
                        builder.append(indent).append(key).append(": \"").append(value).append("\"\n");
                    } else builder.append(indent).append(key).append(": ").append(value).append("\n");
                }

            }
        }
    }
}
