package net.t2code.t2codelib.VELOCITY.api.yml;// This class was created by JaTiTV.


import net.t2code.t2codelib.util.T2C_LanguageEnum;
import net.t2code.t2codelib.util.T2C_ConfigItem;
import org.slf4j.Logger;
import org.yaml.snakeyaml.Yaml;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

public class T2CV_ConfigWriter {

    public static void createConfig(Logger logger, File configFile, T2C_ConfigItem[] manager, String... header) throws IOException {
        boolean exist = configFile.exists();
        if (!exist) {
            configFile.getParentFile().mkdirs();
            try {
                configFile.createNewFile();
            } catch (IOException e) {
                e.printStackTrace();
                return;
            }
        }

        Yaml yaml = new Yaml();
        Map<String, Object> config = yaml.load(Files.newInputStream(configFile.toPath()));
        Map<String, List<String>> comments = new LinkedHashMap<>();
        if (config == null) {
            config = new LinkedHashMap<>();
        }

        T2C_LanguageEnum lang = T2C_LanguageEnum.english;

        for (T2C_ConfigItem item : manager) {
            if (item.getForceSet() || !exist) {
                readValue(config, item.getPath(), item);
                addValue(config, item.getPath(), item.getValue());
                List<String> commandList = item.getComments().get(lang);
                if (commandList != null) {
                    comments.put(item.getPath(), commandList);
                }
            }

        }
        saveConfigWithComments(configFile, comments, config, header);
    }

    private static void readValue(Map<String, Object> config, String path, T2C_ConfigItem value) {
        if (path.contains(".")) {
            String[] pathsplit = path.split("\\.");
            String key = pathsplit[0];
            if (config.containsKey(key)) {
                StringBuilder zw = new StringBuilder();
                for (int i = 1; i < pathsplit.length; i++) {
                    zw.append(pathsplit[i]);
                    if (i != (pathsplit.length - 1)) {
                        zw.append(".");
                    }
                }
                readValue((Map<String, Object>) config.get(key), zw.toString(), value);
            }
        } else {
            Object vl = config.get(path);
            if (vl != null) {
                value.setValue(vl);
            }
        }
    }

    private static void addValue(Map<String, Object> config, String path, Object value) {
        if (path.contains(".")) {
            String[] pathsplit = path.split("\\.");
            String key = pathsplit[0];
            if (config.containsKey(key)) {
                StringBuilder zw = new StringBuilder();
                for (int i = 1; i < pathsplit.length; i++) {
                    zw.append(pathsplit[i]);
                    if (i != (pathsplit.length - 1)) {
                        zw.append(".");
                    }
                }
                addValue((Map<String, Object>) config.get(key), zw.toString(), value);
            } else {
                Map<String, Object> newMap = new LinkedHashMap<>();
                StringBuilder zw = new StringBuilder();
                for (int i = 1; i < pathsplit.length; i++) {
                    zw.append(pathsplit[i]);
                    if (i != (pathsplit.length - 1)) {
                        zw.append(".");
                    }
                }
                addValue(newMap, zw.toString(), value);
                config.put(key, newMap);
            }
        } else {
            if (!config.containsKey(path)) {
                config.put(path, value);
            }
        }
    }


    private static void saveConfigWithComments(File file, Map<String, List<String>> comments, Map<String, Object> config, String... headers) {
        try {
            StringBuilder configContent = new StringBuilder();
            for (String h : headers) {
                configContent.append(h).append("\n");
            }
            configContent.append("\n");
            addSection(config, comments, configContent, "", 0);

            // Write the content to the file
            Files.write(file.toPath(), configContent.toString().getBytes());
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private static void addSection(Map<String, Object> config, Map<String, List<String>> comments, StringBuilder builder, String prefix, int indentLevel) {
        String indent = "  ".repeat(indentLevel);


        for (Map.Entry<String, Object> key : config.entrySet()) {
            String fullKey = prefix.isEmpty() ? key.getKey() : prefix + "." + key.getKey();
            Object value = key.getValue();

            // Add comment if it exists for this key
            List<String> commentList = comments.get(fullKey);
            if (commentList != null) {
                for (String c : commentList) {
                    if (c.isEmpty()) {
                        builder.append(indent).append(c).append("\n");
                    } else builder.append(indent).append("# ").append(c).append("\n");
                }
            }
            // Check if the value is a section (nested map)
            if (value instanceof Map) {
                // Correctly add the section
                builder.append(indent).append(key.getKey()).append(":\n");
                addSection((Map<String, Object>) value, comments, builder, fullKey, indentLevel + 1);
            } else {
                // Add value with proper indentation
                // builder.append(indent).append(key.getKey()).append(": ").append(value).append("\n");
                if (value instanceof List) {
                    List<?> list = (List<?>) value;
                    if (list.size() == 1){
                        for (Object s : list) {
                            if (s instanceof String) {
                                builder.append(indent).append(key.getKey()).append(": \"").append(value).append("\"\n");
                            } else builder.append(indent).append(key.getKey()).append(": ").append(value).append("\n");
                        }
                    }else {
                        builder.append(indent).append(key.getKey()).append(": ").append("\n");
                        for (Object s : list) {
                            if (s instanceof String) {
                                builder.append(indent).append("- \"").append(s).append("\"\n");
                            } else builder.append(indent).append("- ").append(s).append("\n");
                        }
                    }
                } else {
                    if (value instanceof String) {
                        builder.append(indent).append(key.getKey()).append(": \"").append(value).append("\"\n");
                    } else builder.append(indent).append(key.getKey()).append(": ").append(value).append("\n");
                }
            }
        }
    }
}
