package net.t2code.t2codelib.VELOCITY.system.config;

import net.t2code.t2codelib.util.T2C_GenerateFrame;
import net.t2code.t2codelib.util.T2C_LanguageEnum;
import net.t2code.t2codelib.util.T2C_ConfigItem;
import net.t2code.t2codelib.util.T2C_Util;
import net.t2code.t2codelib.VELOCITY.api.messages.T2CV_Send;
import net.t2code.t2codelib.VELOCITY.api.yml.T2CV_ConfigWriter;
import org.slf4j.Logger;

import java.io.File;
import java.io.IOException;
import java.nio.file.Path;
import java.util.HashMap;
import java.util.List;

public class T2CV_LibConfig {

    public enum VALUES implements T2C_ConfigItem {
        updateTimer("updateCheck.timerInMin", 60, true,
                new HashMap<>() {{
                    put(T2C_LanguageEnum.english, List.of("In this option you can set the time interval in minutes in which updates should be checked."));
                }}),
        seePreReleaseUpdates("updateCheck.seePreReleaseUpdates", true, true,
                new HashMap<>() {{
                    put(T2C_LanguageEnum.english, List.of("In this option you can set whether you want to receive and display beta and snapshot versions in the update check."));
                }}),

        debug("plugin.debug.debugModus", false, true,
                new HashMap<>() {{
                    put(T2C_LanguageEnum.english, List.of("The debug mode sends more detailed debug information to the console."));
                }}),

        apiCommandGUIEnable("api.commandGUI.enable", false, true,
                new HashMap<>() {{
                    put(T2C_LanguageEnum.english, List.of("With this option you activate the api interface for the T2C-CommandGUI plugin."));
                }}),
        apiAutoResponse("api.autoResponse.enable", false, true,
                new HashMap<>() {{
                    put(T2C_LanguageEnum.english, List.of("With this option you activate the api interface for the T2C-AutoResponse plugin."));
                }}),
        apiOpSecurity("api.opSecurity.enable", false, true,
                new HashMap<>() {{
                    put(T2C_LanguageEnum.english, List.of("With this option you activate the api interface for the T2C-OPSecurity plugin."));
                }}),
        ;

        private final String path;
        private Object value;
        private final boolean forceSet;
        private final HashMap<T2C_LanguageEnum, List<String>> comments;

        VALUES(String path, Object value, boolean forceSet, HashMap<T2C_LanguageEnum, List<String>> comments) {
            this.path = path;
            this.value = value;
            this.forceSet = forceSet;
            this.comments = comments;
        }

        @Override
        public String getPath() {
            return path;
        }

        @Override
        public Object getValue() {
            return value;
        }

        @Override
        public boolean getForceSet() {
            return forceSet;
        }

        @Override
        public HashMap<T2C_LanguageEnum, List<String>> getComments() {
            return comments;
        }

        @Override
        public String getLanguagePath() {
            return null;
        }

        @Override
        public void setValue(Object newValue) {
            value = newValue;
        }
    }

    public static void set(Path dataDirectory, Logger logger) throws IOException {
        long long_ = System.currentTimeMillis();
        T2CV_ConfigWriter.createConfig(logger, new File(dataDirectory.getParent() + "/T2CodeLib", "config.yml"), VALUES.values(), T2C_Util.getConfigLogo());
        T2CV_Send.info(logger, T2C_GenerateFrame.setCenterAligned("", "The config.yml were successfully created / updated." + " - " + (System.currentTimeMillis() - long_) + "ms"));
    }
}
